package v004::RDF::Source::DBI::Resource;
use strict;
use Carp;
use Data::Dumper;

sub init
{
    my( $self ) = @_;
    $self->{'source'}{'dbh'} = $self->{'model'}->dbh;
}

sub dbh
{
    my( $self ) = @_;
    return $self->{'source'}{'dbh'};
}

sub init_by_id
{
    my( $self, $id ) = @_;

    my $sql = qq(select uri, value, isprefix, lang
		 from resources
		 where id=$id
		 );

    my( $uri, $value, $isprefix, $lang ) = 
	$self->dbh->selectrow_array($sql);

    # Could be an internal id, not contained in the DB
    
    $self->{'simple'}{'uri'}      = $uri || $self->model->ns_local.$id;
    $self->{'simple'}{'value'}    = $value;
    $self->{'simple'}{'lang'}     = $lang;

    $self->{'source'}{'id'}       = $id;
    $self->{'source'}{'isprefix'} = $isprefix;
}

sub init_by_uri
{
    my( $self, $uri ) = @_;
    
    confess "No uri specified" unless $uri;
    my $euri = $self->dbh->quote( $uri );
    my $sql = qq(select id, value, isprefix, lang
		 from resources
		 where uri=$euri
		 );

    my( $id, $value, $isprefix, $lang ) = 
	$self->dbh->selectrow_array($sql);

    # Could be an internal uri, not contained in the DB
    if( not $id )
    {
	my $ns_local = $self->model->ns_local;
	$uri =~ /^$ns_local(\d+)$/ or confess "$uri not found in DB";
	$id = $1;
    }
   
    $self->{'simple'}{'uri'}      = $uri;
    $self->{'simple'}{'value'}    = $value;
    $self->{'simple'}{'lang'}     = $lang;

    $self->{'source'}{'id'}       = $id;
    $self->{'source'}{'isprefix'} = $isprefix;
}


##############################
#
#  Public methods
#
##############################

sub get_arcs_by_subject
{
    my( $self, $pred ) = @_;

    my $subj_id = $self->id;
    
    my $sql;
    if( $pred )
    {
	my $pred_id = $pred->id;
	$sql = qq(select id
		  from statements
		  where subj = $subj_id
		  and pred = $pred_id
		  );
    }
    else
    {
	$sql = qq(select id
		  from statements
		  where subj = $subj_id
		  );
    }

    my @sta_list = ();
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;
    while( my( $id ) = $sth->fetchrow_array )
    {
	push @sta_list, $self->model->get_arc_by_id( $id );
    }
    $sth->finish;

    return \@sta_list;
}

sub get_arcs_by_object
{
    my( $self, $pred ) = @_;

    my $obj_id = $self->id;

    my $sql;
    if( $pred )
    {
	my $pred_id = $pred->id;
	$sql = qq(select id
		  from statements
		  where obj = $obj_id
		  and pred = $pred_id
		  );
    }
    else
    {
	$sql = qq(select id
		  from statements
		  where obj = $obj_id
		  );
    }

    my @sta_list = ();
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;
    while( my( $id ) = $sth->fetchrow_array )
    {
	push @sta_list, $self->model->get_arc_by_id( $id );
    }
    $sth->finish;

    return \@sta_list;
}

sub get_objects
{
    my( $self, $pred ) = @_;

    $pred or confess "Predicate missing";

    my $subj_id = $self->id;
    my $pred_id = $pred->id;
    die unless defined $pred_id and defined $subj_id;

    my $sql = qq(select obj 
		 from statements
		 where subj=$subj_id
		 and pred=$pred_id
		 and fact is not null
		 );
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;

    my @obj_list = ();
    while( my( $id ) = $sth->fetchrow_array )
    {
	push @obj_list, $self->model->get_node_by_id( $id );
    }
    $sth->finish;
    return \@obj_list;
}

sub get_subjects
{
    my( $self, $pred ) = @_;

    $pred or confess "Predicate missing";

    my $pred_id  = $pred->id;
    my $obj_id   = $self->id;
    die unless defined $pred_id and defined $obj_id;

    my $sql = qq(select subj 
		 from statements
		 where pred=$pred_id
		 and obj=$obj_id
		 and fact is not null
		 );
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;

#    die Dumper $sql;

    my @subj_list = ();
    while( my( $id ) = $sth->fetchrow_array )
    {
	push @subj_list, $self->model->get_node_by_id( $id );
    }
    $sth->finish;
    return \@subj_list;
}

sub has_property
{
    my( $self, $pred ) = @_;

    my $pred_id = $pred->id;
    my $subj_id = $self->id;

    my $sql = qq(select id 
		 from statements 
		 where subj = $subj_id
		 and pred = $pred_id
		 );

    my( $id ) = $self->dbh->selectrow_array( $sql );
#    die Dumper $id;

    return $id ? 1 : undef;
}


sub get_membership_objects
{
    my( $self ) = @_;

    my $id = $self->id;
    my $nsrdf = $self->model->ns_rdf;

    my $sql = qq(select uri, obj 
		 from statements, resources
		 where subj=$id
		 and pred=resources.id
		 and uri like '$nsrdf\\\\_%'
		 and fact is not null
		 );
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;

    my @obj_list = ();
    while( my( $uri, $obj ) = $sth->fetchrow_array )
    {
	next unless $uri =~ /^$nsrdf\_(\d+)$/;
	$obj_list[ $1 ] = $self->model->get_node_by_id( $obj );
    }
    $sth->finish;
    return [ grep { $_ if defined } @obj_list ]; # Do not return empty slots
}

sub get_membership_arcs
{
    my( $self ) = @_;

    my $id = $self->id;
    my $nsrdf = $self->model->ns_rdf;

    my $sql = qq(select uri, obj, statements.id
		 from statements, resources
		 where subj=$id
		 and pred=resources.id
		 and uri like '$nsrdf\\\\_%'
		 and fact is not null
		 );
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;

    my @arc_list = ();
    while( my( $uri, $obj, $id ) = $sth->fetchrow_array )
    {
	next unless $uri =~ /^$nsrdf\_(\d+)$/;
	$arc_list[ $1 ] = $self->model->get_arc_by_id( $id );
    }
    $sth->finish;
    return [ grep { $_ if defined } @arc_list ]; # Do not return empty slots
}

sub delete
{
    my( $self ) = @_;

    my $id = $self->id;

    my $sql = qq(delete from resources 
		 where id = $id
		 );

    $self->dbh->do( $sql )
	or die $self->dbh->errstr;

    undef $self;
}

sub store
{
    my( $self ) = @_;

    my $id = $self->id;

    my $uri = $self->dbh->quote( $self->uri );
    my $lang = $self->dbh->quote( $self->lang );
    my $value = $self->dbh->quote( $self->value );

    my $sql = qq[update resources set
		 uri=$uri, lang=$lang, value=$value
		 where id=$id
		 ];
    
    $self->dbh->do( $sql )
	or die $self->dbh->errstr;
}

sub id       { $_[0]->{'source'}{'id'} }
sub isprefix { $_[0]->{'source'}{'isprefix'} }


1;





